@@ -2,6 +2,7 @@
 #include "gdkwaylandcolor-private.h"
 #include "gdksurface-wayland-private.h"
 #include <gdk/wayland/color-management-v1-client-protocol.h>
+#include <gdk/wayland/color-representation-v1-client-protocol.h>
 
 typedef struct _ImageDescription ImageDescription;
 
@@ -27,7 +28,7 @@ wl_to_cicp_primaries (enum wp_color_manager_v1_primaries cp)
 static enum wp_color_manager_v1_primaries
 cicp_to_wl_primaries (uint cp)
 {
-  for (guint i = 0; i < G_N_ELEMENTS (primaries_map); i++)
+  for (guint i = WP_COLOR_MANAGER_V1_PRIMARIES_SRGB; i < G_N_ELEMENTS (primaries_map); i++)
     if (primaries_map[i] == cp)
        return (enum wp_color_manager_v1_primaries)i;
 
@@ -59,7 +60,7 @@ primaries_to_wl_primaries (const uint primaries[8],
 {
   guint i, j;
 
-  for (i = 0; i < G_N_ELEMENTS (primaries_primaries); i++)
+  for (i = WP_COLOR_MANAGER_V1_PRIMARIES_SRGB; i < G_N_ELEMENTS (primaries_primaries); i++)
     {
       for (j = 0; j < 8; j++)
         {
@@ -100,13 +101,99 @@ wl_to_cicp_transfer (enum wp_color_manager_v1_transfer_function tf)
 static enum wp_color_manager_v1_transfer_function
 cicp_to_wl_transfer (uint tf)
 {
-  for (guint i = 0; i < G_N_ELEMENTS (transfer_map); i++)
+  for (guint i = WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_BT1886; i < G_N_ELEMENTS (transfer_map); i++)
     if (transfer_map[i] == tf)
        return (enum wp_color_manager_v1_transfer_function)i;
 
   return 0;
 }
 
+struct {
+  enum wp_color_representation_surface_v1_coefficients wp;
+  uint cicp;
+  const char *name;
+} coefficients_map[] = {
+ { .wp = WP_COLOR_REPRESENTATION_SURFACE_V1_COEFFICIENTS_IDENTITY, .cicp = 0, .name = "identity" },
+ { .wp = WP_COLOR_REPRESENTATION_SURFACE_V1_COEFFICIENTS_BT709, .cicp = 1, .name = "bt709" },
+ { .wp = WP_COLOR_REPRESENTATION_SURFACE_V1_COEFFICIENTS_FCC, .cicp = 4, .name = "fcc" },
+ { .wp = WP_COLOR_REPRESENTATION_SURFACE_V1_COEFFICIENTS_BT601, .cicp = 5, .name = "bt601" },
+ { .wp = WP_COLOR_REPRESENTATION_SURFACE_V1_COEFFICIENTS_SMPTE240, .cicp = 7, .name = "smpte240" },
+ { .wp = WP_COLOR_REPRESENTATION_SURFACE_V1_COEFFICIENTS_BT2020, .cicp = 9, .name = "bt2020" },
+ { .wp = WP_COLOR_REPRESENTATION_SURFACE_V1_COEFFICIENTS_BT2020_CL, .cicp = 10, .name = "bt2020-cl" },
+ { .wp = WP_COLOR_REPRESENTATION_SURFACE_V1_COEFFICIENTS_ICTCP, .cicp = 14, .name = "ictcp" },
+};
+
+static enum wp_color_representation_surface_v1_coefficients
+cicp_to_wl_coefficients (uint m)
+{
+  if (m == 6)
+    m = 5;
+
+  for (guint i = 0; i < G_N_ELEMENTS (coefficients_map); i++)
+    if (coefficients_map[i].cicp == m)
+       return coefficients_map[i].wp;
+
+  return 0;
+}
+
+static const char *
+wl_coefficients_name (enum wp_color_representation_surface_v1_coefficients value)
+{
+  for (guint i = 0; i < G_N_ELEMENTS (coefficients_map); i++)
+    if (coefficients_map[i].wp == value)
+       return coefficients_map[i].name;
+
+  return "invalid";
+}
+
+struct {
+  enum wp_color_representation_surface_v1_alpha_mode wp;
+  const char *name;
+} alpha_modes[] = {
+  { .wp = WP_COLOR_REPRESENTATION_SURFACE_V1_ALPHA_MODE_PREMULTIPLIED_ELECTRICAL, .name = "premultiplied-electrical" },
+  { .wp = WP_COLOR_REPRESENTATION_SURFACE_V1_ALPHA_MODE_PREMULTIPLIED_OPTICAL, .name = "premultiplied-optical" },
+  { .wp = WP_COLOR_REPRESENTATION_SURFACE_V1_ALPHA_MODE_STRAIGHT, .name = "straight" },
+};
+
+static const char *
+wl_alpha_name (enum wp_color_representation_surface_v1_alpha_mode alpha)
+{
+  for (guint i = 0; i < G_N_ELEMENTS (alpha_modes); i++)
+    if (alpha_modes[i].wp == alpha)
+       return alpha_modes[i].name;
+
+  return "invalid";
+}
+
+static enum wp_color_representation_surface_v1_alpha_mode
+gdk_alpha_to_wl_alpha (GdkMemoryAlpha alpha)
+{
+  switch (alpha)
+    {
+    case GDK_MEMORY_ALPHA_PREMULTIPLIED:
+    case GDK_MEMORY_ALPHA_OPAQUE:
+      return WP_COLOR_REPRESENTATION_SURFACE_V1_ALPHA_MODE_PREMULTIPLIED_ELECTRICAL;
+    case GDK_MEMORY_ALPHA_STRAIGHT:
+      return WP_COLOR_REPRESENTATION_SURFACE_V1_ALPHA_MODE_STRAIGHT;
+    default:
+      g_assert_not_reached ();
+    }
+}
+
+static const char *
+wl_range_name (enum wp_color_representation_surface_v1_range range)
+{
+  switch (range)
+    {
+    case WP_COLOR_REPRESENTATION_SURFACE_V1_RANGE_FULL:
+      return "full";
+    case WP_COLOR_REPRESENTATION_SURFACE_V1_RANGE_LIMITED:
+      return "limited";
+    default:
+      g_assert_not_reached ();
+    }
+}
+
 struct _GdkWaylandColor
 {
   GdkWaylandDisplay *display;
@@ -121,6 +208,13 @@ struct _GdkWaylandColor
 
   GHashTable *cs_to_desc; /* GdkColorState => wp_image_description_v1 or NULL */
   GHashTable *id_to_cs; /* uint32 identifier => GdkColorState */
+
+  struct wp_color_representation_manager_v1 *color_representation_manager;
+  struct {
+    unsigned int alpha_modes;
+    unsigned int coefficients_limited;
+    unsigned int coefficients_full;
+  } color_representation_supported;
 };
 
 static guint
@@ -216,16 +310,14 @@ static struct wp_color_manager_v1_listener color_manager_listener = {
 };
 
 GdkWaylandColor *
-gdk_wayland_color_new (GdkWaylandDisplay  *display,
-                       struct wl_registry *registry,
-                       uint32_t            id,
-                       uint32_t            version)
+gdk_wayland_color_new (GdkWaylandDisplay *display)
 {
   GdkWaylandColor *color;
 
   color = g_new0 (GdkWaylandColor, 1);
 
   color->display = display;
+
   color->cs_to_desc = g_hash_table_new_full (color_state_hash,
                                              color_state_equal,
                                              (GDestroyNotify) gdk_color_state_unref,
@@ -235,6 +327,15 @@ gdk_wayland_color_new (GdkWaylandDisplay  *display,
                                            NULL,
                                            (GDestroyNotify) gdk_color_state_unref);
 
+  return color;
+}
+
+void
+gdk_wayland_color_set_color_manager (GdkWaylandColor    *color,
+                                     struct wl_registry *registry,
+                                     uint32_t            id,
+                                     uint32_t            version)
+{
   color->color_manager = wl_registry_bind (registry,
                                            id,
                                            &wp_color_manager_v1_interface,
@@ -243,8 +344,6 @@ gdk_wayland_color_new (GdkWaylandDisplay  *display,
   wp_color_manager_v1_add_listener (color->color_manager,
                                     &color_manager_listener,
                                     color);
-
-  return color;
 }
 
 void
@@ -255,6 +354,8 @@ gdk_wayland_color_free (GdkWaylandColor *color)
   g_hash_table_unref (color->cs_to_desc);
   g_hash_table_unref (color->id_to_cs);
 
+  g_clear_pointer (&color->color_representation_manager, wp_color_representation_manager_v1_destroy);
+
   g_free (color);
 }
 
@@ -264,6 +365,12 @@ gdk_wayland_color_get_color_manager (GdkWaylandColor *color)
   return (struct wl_proxy *) color->color_manager;
 }
 
+struct wl_proxy *
+gdk_wayland_color_get_color_representation_manager (GdkWaylandColor *color)
+{
+  return (struct wl_proxy *) color->color_representation_manager;
+}
+
 typedef struct _CsImageDescListenerData {
   GdkWaylandColor *color;
   GdkColorState   *color_state;
@@ -271,6 +378,69 @@ typedef struct _CsImageDescListenerData {
   gboolean         done;
 } CsImageDescListenerData;
 
+static void
+wp_color_representation_manager_v1_supported_alpha_mode (void *data,
+                                                         struct wp_color_representation_manager_v1 *wp_color_representation_manager_v1,
+                                                         uint32_t alpha_mode)
+{
+  GdkWaylandColor *color = data;
+
+  color->color_representation_supported.alpha_modes |= 1 << alpha_mode;
+}
+
+static void
+wp_color_representation_manager_v1_supported_coefficients_and_ranges (void *data,
+                                                                      struct wp_color_representation_manager_v1 *wp_color_representation_manager_v1,
+                                                                      uint32_t coefficients,
+                                                                      uint32_t range)
+{
+  GdkWaylandColor *color = data;
+
+  switch (range)
+    {
+    case WP_COLOR_REPRESENTATION_SURFACE_V1_RANGE_FULL:
+      color->color_representation_supported.coefficients_full |= 1 << coefficients;
+      break;
+
+    case WP_COLOR_REPRESENTATION_SURFACE_V1_RANGE_LIMITED:
+      color->color_representation_supported.coefficients_limited |= 1 << coefficients;
+      break;
+
+    default:
+      g_assert_not_reached ();
+    }
+}
+
+static void
+wp_color_representation_manager_v1_done (void *data,
+                                         struct wp_color_representation_manager_v1 *wp_color_representation_manager_v1)
+{
+}
+
+static struct wp_color_representation_manager_v1_listener color_representation_manager_listener = {
+  wp_color_representation_manager_v1_supported_alpha_mode,
+  wp_color_representation_manager_v1_supported_coefficients_and_ranges,
+  wp_color_representation_manager_v1_done,
+};
+
+void
+gdk_wayland_color_set_color_representation (GdkWaylandColor        *color,
+                                            struct wl_registry     *registry,
+                                            uint32_t                id,
+                                            uint32_t                version)
+{
+  g_assert (color->color_representation_manager == NULL);
+
+  color->color_representation_manager = wl_registry_bind (registry,
+                                                         id,
+                                                         &wp_color_representation_manager_v1_interface,
+                                                         MIN (version, 1));
+
+  wp_color_representation_manager_v1_add_listener (color->color_representation_manager,
+                                                   &color_representation_manager_listener,
+                                                   color);
+}
+
 static void
 cs_image_listener_data_free (CsImageDescListenerData *csi)
 {
@@ -337,12 +507,13 @@ create_image_desc (GdkWaylandColor *color,
   cicp = gdk_color_state_get_cicp (cs);
   if (!cicp)
     {
-      GDK_DEBUG (MISC, "Unsupported color state %s: Not a CICP colorstate",
-                 gdk_color_state_get_name (cs));
+      GDK_DISPLAY_DEBUG (GDK_DISPLAY (color->display), MISC,
+                         "Unsupported color state %s: Not a CICP colorstate",
+                         gdk_color_state_get_name (cs));
       g_hash_table_insert (color->cs_to_desc, gdk_color_state_ref (cs), NULL);
       return;
     }
-  
+
   gdk_cicp_normalize (cicp, &norm);
   primaries = cicp_to_wl_primaries (norm.color_primaries);
   tf = cicp_to_wl_transfer (norm.transfer_function);
@@ -351,12 +522,15 @@ create_image_desc (GdkWaylandColor *color,
        (color->color_manager_supported.features & (1 << WP_COLOR_MANAGER_V1_FEATURE_SET_PRIMARIES)) == 0) ||
       (color->color_manager_supported.transfers & (1 << tf)) == 0)
     {
-      GDK_DEBUG (MISC, "Unsupported color state %s: Primaries or transfer function unsupported",
-                 gdk_color_state_get_name (cs));
+      GDK_DISPLAY_DEBUG (GDK_DISPLAY (color->display), MISC,
+                         "Unsupported color state %s: Primaries or transfer function unsupported",
+                         gdk_color_state_get_name (cs));
       g_hash_table_insert (color->cs_to_desc, gdk_color_state_ref (cs), NULL);
       return;
     }
 
+  g_assert (color->color_manager != NULL);
+
   data.color = color;
   data.color_state = cs;
   data.sync = sync;
@@ -384,7 +558,7 @@ create_image_desc (GdkWaylandColor *color,
   if (sync)
     {
       struct wl_event_queue *event_queue;
-      
+
       event_queue = wl_display_create_queue (color->display->wl_display);
       wl_proxy_set_queue ((struct wl_proxy *) desc, event_queue);
       wp_image_description_v1_add_listener (desc, &cs_image_desc_listener, &data);
@@ -404,51 +578,116 @@ gdk_wayland_color_prepare (GdkWaylandColor *color)
 {
   if (color->color_manager)
     {
-      const char *intents[] = {
-        "perceptual", "relative", "saturation", "absolute", "relative-bpc"
-      };
-      const char *features[] = {
-        "icc-v4-v4", "parametric", "set-primaries", "set-tf-power",
-        "set-mastering-display-primaries", "extended-target-volume"
-      };
-      const char *tf[] = {
-        "bt709", "gamma22", "gamma28", "st240", "linear", "log100",
-        "log316", "xvycc", "bt1361", "srgb", "ext-srgb", "pq", "st428", "hlg"
-      };
-      const char *primaries[] = {
-        "srgb", "pal-m", "pal", "ntsc", "generic-film", "bt2020", "xyz",
-        "dci-p3", "display-p3", "adobe-rgb",
-      };
-
-     for (int i = 0; i < G_N_ELEMENTS (intents); i++)
+      if (GDK_DISPLAY_DEBUG_CHECK (GDK_DISPLAY (color->display), MISC))
         {
-          GDK_DEBUG (MISC, "Rendering intent %d (%s): %s",
-                   i, intents[i], color->color_manager_supported.intents & (1 << i) ? "✓" : "✗");
-        }
+          struct {
+            const char *name;
+            enum wp_color_manager_v1_render_intent value;
+          } intents[] = {
+            { "perceptual", WP_COLOR_MANAGER_V1_RENDER_INTENT_PERCEPTUAL },
+            { "relative", WP_COLOR_MANAGER_V1_RENDER_INTENT_RELATIVE },
+            { "saturation", WP_COLOR_MANAGER_V1_RENDER_INTENT_SATURATION },
+            { "absolute", WP_COLOR_MANAGER_V1_RENDER_INTENT_ABSOLUTE },
+            { "relative-bpc", WP_COLOR_MANAGER_V1_RENDER_INTENT_RELATIVE_BPC },
+          };
+          struct {
+            const char *name;
+            enum wp_color_manager_v1_feature value;
+          } features[] = {
+            { "icc-v2-v4", WP_COLOR_MANAGER_V1_FEATURE_ICC_V2_V4 },
+            { "parametric", WP_COLOR_MANAGER_V1_FEATURE_PARAMETRIC },
+            { "set-primaries", WP_COLOR_MANAGER_V1_FEATURE_SET_PRIMARIES },
+            { "set-tf-power", WP_COLOR_MANAGER_V1_FEATURE_SET_TF_POWER },
+            { "set-luminances", WP_COLOR_MANAGER_V1_FEATURE_SET_LUMINANCES },
+            { "set-mastering-display-primaries", WP_COLOR_MANAGER_V1_FEATURE_SET_MASTERING_DISPLAY_PRIMARIES },
+            { "extended-target-volume", WP_COLOR_MANAGER_V1_FEATURE_EXTENDED_TARGET_VOLUME },
+            { "windows-scrgb", WP_COLOR_MANAGER_V1_FEATURE_WINDOWS_SCRGB },
+          };
+          struct {
+            const char *name;
+            enum wp_color_manager_v1_primaries value;
+          } primaries[] = {
+            { "srgb", WP_COLOR_MANAGER_V1_PRIMARIES_SRGB },
+            { "pal-m", WP_COLOR_MANAGER_V1_PRIMARIES_PAL_M },
+            { "pal", WP_COLOR_MANAGER_V1_PRIMARIES_PAL },
+            { "ntsc", WP_COLOR_MANAGER_V1_PRIMARIES_NTSC },
+            { "generic-film", WP_COLOR_MANAGER_V1_PRIMARIES_GENERIC_FILM },
+            { "bt2020", WP_COLOR_MANAGER_V1_PRIMARIES_BT2020 },
+            { "cie1931-xyz", WP_COLOR_MANAGER_V1_PRIMARIES_CIE1931_XYZ },
+            { "dci-p3", WP_COLOR_MANAGER_V1_PRIMARIES_DCI_P3 },
+            { "display-p3", WP_COLOR_MANAGER_V1_PRIMARIES_DISPLAY_P3 },
+            { "adobe-rgb", WP_COLOR_MANAGER_V1_PRIMARIES_ADOBE_RGB },
+          };
+          struct {
+            const char *name;
+            enum wp_color_manager_v1_transfer_function value;
+          } tf[] = {
+            { "bt1886", WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_BT1886 },
+            { "gamma22", WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_GAMMA22 },
+            { "gamma28", WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_GAMMA28 },
+            { "st240", WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_ST240 },
+            { "ext-linear", WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_EXT_LINEAR },
+            { "log100", WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_LOG_100 },
+            { "log316", WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_LOG_316 },
+            { "xvycc", WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_XVYCC },
+            { "srgb", WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_SRGB },
+            { "ext-srgb", WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_EXT_SRGB },
+            { "st2084-pq", WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_ST2084_PQ },
+            { "st428", WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_ST428 },
+            { "hlg", WP_COLOR_MANAGER_V1_TRANSFER_FUNCTION_HLG },
+          };
+          unsigned int len;
 
-      for (int i = 0; i < G_N_ELEMENTS (features); i++)
-        {
-          GDK_DEBUG (MISC, "Feature %d (%s): %s",
-                   i, features[i], color->color_manager_supported.features & (1 << i) ? "✓" : "✗");
-        }
+         len = 0;
+         for (int i = 0; i < G_N_ELEMENTS (intents); i++)
+           len = MAX (len, strlen (intents[i].name));
 
-      for (int i = 0; i < G_N_ELEMENTS (tf); i++)
-        {
-          GDK_DEBUG (MISC, "Transfer function %d (%s): %s",
-                   i, tf[i], color->color_manager_supported.transfers & (1 << i) ? "✓" : "✗");
-        }
+         for (int i = 0; i < G_N_ELEMENTS (intents); i++)
+            gdk_debug_message ("Rendering intent %2u (%s): %*s%s",
+                               intents[i].value,
+                               intents[i].name,
+                               (int) (len - strlen (intents[i].name)), "",
+                               color->color_manager_supported.intents & (1 << intents[i].value) ? "✓" : "✗");
 
-      for (int i = 0; i < G_N_ELEMENTS (primaries); i++)
-        {
-          GDK_DEBUG (MISC, "Primaries %d (%s): %s",
-                   i, primaries[i], color->color_manager_supported.primaries& (1 << i) ? "✓" : "✗");
+         len = 0;
+         for (int i = 0; i < G_N_ELEMENTS (features); i++)
+           len = MAX (len, strlen (features[i].name));
+
+          for (int i = 0; i < G_N_ELEMENTS (features); i++)
+            gdk_debug_message ("Feature %2u (%s): %*s%s",
+                               features[i].value,
+                               features[i].name,
+                               (int) (len - strlen (features[i].name)), "",
+                               color->color_manager_supported.features & (1 << features[i].value) ? "✓" : "✗");
+
+         len = 0;
+         for (int i = 0; i < G_N_ELEMENTS (primaries); i++)
+           len = MAX (len, strlen (primaries[i].name));
+
+          for (int i = 0; i < G_N_ELEMENTS (primaries); i++)
+            gdk_debug_message ("Primaries %2u (%s): %*s%s",
+                               primaries[i].value,
+                               primaries[i].name,
+                               (int) (len - strlen (primaries[i].name)), "",
+                               color->color_manager_supported.primaries & (1 << primaries[i].value) ? "✓" : "✗");
+
+         len = 0;
+         for (int i = 0; i < G_N_ELEMENTS (tf); i++)
+           len = MAX (len, strlen (tf[i].name));
+
+          for (int i = 0; i < G_N_ELEMENTS (tf); i++)
+            gdk_debug_message ("Transfer function %2u (%s): %*s%s",
+                               tf[i].value,
+                               tf[i].name,
+                               (int) (len - strlen (tf[i].name)), "",
+                               color->color_manager_supported.transfers & (1 << tf[i].value) ? "✓" : "✗");
         }
     }
 
   if (color->color_manager &&
       !(color->color_manager_supported.intents & (1 << WP_COLOR_MANAGER_V1_RENDER_INTENT_PERCEPTUAL)))
     {
-      GDK_DEBUG (MISC, "Not using color management: Missing perceptual render intent");
+      GDK_DISPLAY_DEBUG (GDK_DISPLAY (color->display), MISC, "Not using color management: Missing perceptual render intent");
       g_clear_pointer (&color->color_manager, wp_color_manager_v1_destroy);
     }
 
@@ -459,7 +698,7 @@ gdk_wayland_color_prepare (GdkWaylandColor *color)
          (color->color_manager_supported.features & (1 << WP_COLOR_MANAGER_V1_FEATURE_SET_PRIMARIES)))))
 
     {
-      GDK_DEBUG (MISC, "Not using color management: Can't create srgb image description");
+      GDK_DISPLAY_DEBUG (GDK_DISPLAY (color->display), MISC, "Not using color management: Can't create srgb image description");
       g_clear_pointer (&color->color_manager, wp_color_manager_v1_destroy);
     }
 
@@ -481,17 +720,64 @@ gdk_wayland_color_prepare (GdkWaylandColor *color)
         }
     }
 
-  return color->color_manager != NULL;
+  if (color->color_representation_manager)
+    {
+      if (GDK_DISPLAY_DEBUG_CHECK (GDK_DISPLAY (color->display), MISC))
+        {
+         unsigned int len;
+
+         len = 0;
+         for (int i = 0; i < G_N_ELEMENTS (alpha_modes); i++)
+           len = MAX (len, strlen (alpha_modes[i].name));
+
+         for (int i = 0; i < G_N_ELEMENTS (alpha_modes); i++)
+           gdk_debug_message ("Alpha mode %2u (%s): %*s%s",
+                              alpha_modes[i].wp,
+                              alpha_modes[i].name,
+                              (int) (len - strlen (alpha_modes[i].name)), "",
+                              color->color_representation_supported.alpha_modes & (1 << alpha_modes[i].wp) ? "✓" : "✗");
+
+         len = 0;
+         for (int i = 0; i < G_N_ELEMENTS (coefficients_map); i++)
+           len = MAX (len, strlen (coefficients_map[i].name));
+
+         for (int i = 0; i < G_N_ELEMENTS (coefficients_map); i++)
+           gdk_debug_message ("Coefficients (full range) %2u (%s): %*s%s",
+                              coefficients_map[i].wp,
+                              coefficients_map[i].name,
+                              (int) (len - strlen (coefficients_map[i].name)), "",
+                              color->color_representation_supported.coefficients_full & (1 << coefficients_map[i].wp) ? "✓" : "✗");
+
+         for (int i = 0; i < G_N_ELEMENTS (coefficients_map); i++)
+           gdk_debug_message ("Coefficients (limited range) %2u (%s): %*s%s",
+                              coefficients_map[i].wp,
+                              coefficients_map[i].name,
+                              (int) (len - strlen (coefficients_map[i].name)), "",
+                              color->color_representation_supported.coefficients_limited & (1 << coefficients_map[i].wp) ? "✓" : "✗");
+        }
+    }
+
+  if (color->color_representation_manager &&
+      !(color->color_representation_supported.coefficients_full & (1 << WP_COLOR_REPRESENTATION_SURFACE_V1_COEFFICIENTS_IDENTITY)))
+    {
+      GDK_DISPLAY_DEBUG (GDK_DISPLAY (color->display), MISC, "Not using color representation: Can't create identity transform");
+      g_clear_pointer (&color->color_representation_manager, wp_color_representation_manager_v1_destroy);
+    }
+
+  return color->color_manager != NULL || color->color_representation_manager != NULL;
 }
 
 struct _GdkWaylandColorSurface
 {
   GdkWaylandColor *color;
-  struct wp_color_management_surface_v1 *surface;
-  struct wp_color_management_surface_feedback_v1 *feedback;
+  struct wl_surface *wl_surface;
+  struct wp_color_management_surface_v1 *mgmt_surface;
+  struct wp_color_management_surface_feedback_v1 *mgmt_feedback;
   ImageDescription *current_desc;
   GdkColorStateChanged callback;
   gpointer data;
+
+  struct wp_color_representation_surface_v1 *repr_surface;
 };
 
 struct _ImageDescription
@@ -810,7 +1096,7 @@ preferred_changed (void *data,
   desc->surface = self;
   self->current_desc = desc;
 
-  desc->image_desc = wp_color_management_surface_feedback_v1_get_preferred_parametric (self->feedback);
+  desc->image_desc = wp_color_management_surface_feedback_v1_get_preferred_parametric (self->mgmt_feedback);
 
   wp_image_description_v1_add_listener (desc->image_desc, &image_desc_listener, desc);
 }
@@ -830,15 +1116,19 @@ gdk_wayland_color_surface_new (GdkWaylandColor      *color,
   self = g_new0 (GdkWaylandColorSurface, 1);
 
   self->color = color;
+  self->wl_surface = wl_surface;
 
-  self->surface = wp_color_manager_v1_get_surface (color->color_manager, wl_surface);
-  self->feedback = wp_color_manager_v1_get_surface_feedback (color->color_manager, wl_surface);
+  if (color->color_manager)
+    {
+      self->mgmt_surface = wp_color_manager_v1_get_surface (color->color_manager, wl_surface);
+      self->mgmt_feedback = wp_color_manager_v1_get_surface_feedback (color->color_manager, wl_surface);
 
-  self->callback = callback;
-  self->data = data;
+      self->callback = callback;
+      self->data = data;
 
-  wp_color_management_surface_feedback_v1_add_listener (self->feedback, &color_listener, self);
-  preferred_changed (self, self->feedback, 0);
+      wp_color_management_surface_feedback_v1_add_listener (self->mgmt_feedback, &color_listener, self);
+      preferred_changed (self, self->mgmt_feedback, 0);
+    }
 
   return self;
 }
@@ -848,8 +1138,9 @@ gdk_wayland_color_surface_free (GdkWaylandColorSurface *self)
 {
   gdk_wayland_color_surface_clear_image_desc (self);
 
-  wp_color_management_surface_v1_destroy (self->surface);
-  wp_color_management_surface_feedback_v1_destroy (self->feedback);
+  g_clear_pointer (&self->mgmt_surface, wp_color_management_surface_v1_destroy);
+  g_clear_pointer (&self->mgmt_feedback, wp_color_management_surface_feedback_v1_destroy);
+  g_clear_pointer (&self->repr_surface, wp_color_representation_surface_v1_destroy);
 
   g_free (self);
 }
@@ -862,36 +1153,186 @@ gdk_wayland_color_get_image_description (GdkWaylandColor *color,
 
   if (g_hash_table_lookup_extended (color->cs_to_desc, cs, NULL, &result))
     return result;
-  
+
   create_image_desc (color, cs, TRUE);
 
   if (!g_hash_table_lookup_extended (color->cs_to_desc, cs, NULL, &result))
     {
       g_assert_not_reached ();
     }
-  
+
   return result;
 }
 
+static gboolean
+gdk_wayland_color_get_color_representation (GdkWaylandColor *color,
+                                            GdkColorState   *cs,
+                                            GdkMemoryFormat  format,
+                                            uint32_t        *coefficients,
+                                            uint32_t        *range,
+                                            uint32_t        *alpha)
+{
+  GdkCicpParams *cicp = gdk_color_state_create_cicp_params (cs);
+  uint32_t c, a;
+
+  *coefficients = 0;
+  *range = 0;
+  *alpha = 0;
+
+  c = cicp_to_wl_coefficients (gdk_cicp_params_get_matrix_coefficients (cicp));
+
+  if (gdk_cicp_params_get_range (cicp) == GDK_CICP_RANGE_NARROW)
+    {
+      *range = WP_COLOR_REPRESENTATION_SURFACE_V1_RANGE_LIMITED;
+      if (color->color_representation_supported.coefficients_limited & (1 << c))
+        *coefficients = c;
+      else
+        return FALSE;
+    }
+  else
+    {
+      *range = WP_COLOR_REPRESENTATION_SURFACE_V1_RANGE_FULL;
+      if (color->color_representation_supported.coefficients_full & (1 << c))
+        *coefficients = c;
+      else
+        return FALSE;
+    }
+
+  a = gdk_alpha_to_wl_alpha (gdk_memory_format_alpha (format));
+  if (color->color_representation_supported.alpha_modes & (1 << a))
+    *alpha = a;
+  else
+    return FALSE;
+
+  return TRUE;
+}
+
 void
 gdk_wayland_color_surface_set_color_state (GdkWaylandColorSurface *self,
-                                           GdkColorState          *cs)
+                                           GdkColorState          *cs,
+                                           guint32                 fourcc,
+                                           gboolean                premultiplied)
 {
-  struct wp_image_description_v1 *desc;
+  if (self->mgmt_surface)
+    {
+      struct wp_image_description_v1 *desc;
 
-  desc = gdk_wayland_color_get_image_description (self->color, cs);
+      desc = gdk_wayland_color_get_image_description (self->color, cs);
 
-  if (desc)
-    wp_color_management_surface_v1_set_image_description (self->surface,
-                                                          desc,
-                                                          WP_COLOR_MANAGER_V1_RENDER_INTENT_PERCEPTUAL);
-  else
-    wp_color_management_surface_v1_unset_image_description (self->surface);
+      g_assert (desc);
+
+      GDK_DISPLAY_DEBUG (GDK_DISPLAY (self->color->display), MISC,
+                         "Setting color state %s (fourcc %.4s, premul %d) on surface: image desc %p",
+                         gdk_color_state_get_name (cs),
+                         (char *) &fourcc,
+                         premultiplied,
+                         desc);
+
+      wp_color_management_surface_v1_set_image_description (self->mgmt_surface,
+                                                            desc,
+                                                            WP_COLOR_MANAGER_V1_RENDER_INTENT_PERCEPTUAL);
+    }
+
+  if (self->color->color_representation_manager)
+    {
+      uint32_t coefficients, range, alpha;
+      gboolean ret G_GNUC_UNUSED;
+      GdkMemoryFormat format;
+      gboolean is_yuv;
+
+      ret = gdk_memory_format_find_by_dmabuf_fourcc (fourcc, premultiplied, &format, &is_yuv);
+      g_assert (ret);
+
+      ret = gdk_wayland_color_get_color_representation (self->color, cs, format, &coefficients, &range, &alpha);
+      g_assert (ret);
+
+      GDK_DISPLAY_DEBUG (GDK_DISPLAY (self->color->display), MISC,
+                         "Setting color state %s (fourcc %.4s, premul %d)  on surface: coefficients: %u (%s), range: %u (%s), alpha %u (%s)",
+                         gdk_color_state_get_name (cs),
+                         (char *) &fourcc,
+                         premultiplied,
+                         coefficients, wl_coefficients_name (coefficients),
+                         range, wl_range_name (range),
+                         alpha, wl_alpha_name (alpha));
+
+      if (!self->repr_surface)
+        self->repr_surface= wp_color_representation_manager_v1_get_surface (self->color->color_representation_manager, self->wl_surface);
+
+      wp_color_representation_surface_v1_set_coefficients_and_range (self->repr_surface, coefficients, range);
+      wp_color_representation_surface_v1_set_alpha_mode (self->repr_surface, alpha);
+    }
+}
+
+void
+gdk_wayland_color_surface_unset_color_state (GdkWaylandColorSurface *self)
+{
+  if (self->mgmt_surface)
+    wp_color_management_surface_v1_unset_image_description (self->mgmt_surface);
+
+  g_clear_pointer (&self->repr_surface, wp_color_representation_surface_v1_destroy);
 }
 
 gboolean
-gdk_wayland_color_surface_can_set_color_state (GdkWaylandColorSurface *self,
-                                               GdkColorState          *cs)
+gdk_wayland_color_surface_can_set_color_state (GdkWaylandColorSurface  *self,
+                                               GdkColorState           *cs,
+                                               guint32                  fourcc,
+                                               gboolean                 premultiplied,
+                                               GError                 **error)
 {
-  return gdk_wayland_color_get_image_description (self->color, cs) != NULL;
+  GdkMemoryFormat format;
+  gboolean is_yuv;
+  GdkColorState *default_cs;
+
+  if (!gdk_memory_format_find_by_dmabuf_fourcc (fourcc, premultiplied, &format, &is_yuv))
+    {
+      g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED, "No memory format for fourcc");
+      return FALSE;
+    }
+
+  default_cs = is_yuv ? GDK_COLOR_STATE_YUV : GDK_COLOR_STATE_SRGB;
+
+  if (!self->mgmt_surface)
+    {
+      if (!gdk_color_state_equivalent (cs, default_cs))
+        {
+          g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED, "No color-management, non-default color state");
+          return FALSE;
+        }
+    }
+  else
+    {
+      if (!gdk_wayland_color_get_image_description (self->color, cs))
+        {
+          g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED, "No image description");
+          return FALSE;
+        }
+    }
+
+  if (!self->color->color_representation_manager)
+    {
+      if (!gdk_color_state_equivalent (cs, default_cs))
+        {
+          g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED, "No color-representation, non-default color state");
+          return FALSE;
+        }
+
+      if (gdk_memory_format_alpha (format) == GDK_MEMORY_ALPHA_STRAIGHT)
+        {
+          g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED, "No color-representation, straight alpha");
+          return FALSE;
+        }
+    }
+  else
+    {
+      uint32_t coefficients, range, alpha;
+
+      if (!gdk_wayland_color_get_color_representation (self->color, cs, format, &coefficients, &range, &alpha))
+        {
+          g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED, coefficients == 0 ? "Coefficients not supported"
+                                                                                       : "Alpha not supported");
+          return FALSE;
+        }
+    }
+
+  return TRUE;
 }
